/***************************************************************************
 *****                                                                 *****
 *****  Name: UNIT1.CPP, part of IO08.CPP                              *****
 *****  Ver.: 2.1                                                      *****
 *****  Date: 11-JAN-2001                                              *****
 *****  Func: Win98/2000 frontend for USB08 evaluation board           *****
 *****  Auth: Andreas Dannenberg, eMail: adannenb@elektronikladen.de   *****
 *****  Rem.: Build with Borland C++ Builder Pro V5.0                  *****
 *****                                                                 *****
 ***************************************************************************/

#define UseUSBpipe                     // by defining or NOT defining this
                                       // label before compiling, you can
                                       // select the port to use
                                       // (if defined => USB, if not => RS232)

//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "Unit1.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "CGAUGES"
#pragma resource "*.dfm"
TMainForm *MainForm;
//---------------------------------------------------------------------------
 __fastcall TMainForm::TMainForm(TComponent* Owner): TForm(Owner)
{
#ifdef UseUSBpipe                                     // show the type of
  MainForm->Caption = MainForm->Caption + " (USB)";   // interface used
#else                                                 // in window title
  MainForm->Caption = MainForm->Caption + " (RS232)";
#endif

  if (openpipe())                                     // start pipe
    Timer1->Enabled = TRUE;                           // if OK, start transfer
  else
    StatusBar->SimpleText = "Couldn't open port! Please restart application to retry!";
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FormDestroy(TObject *Sender)
{
  closepipe();                                        // stop pipe
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FormShow(TObject *Sender)
{
  Images->GetBitmap(LED_OFF, LED1->Picture->Bitmap); // set-up inital
  Images->GetBitmap(LED_OFF, LED2->Picture->Bitmap); // LED-images
  Images->GetBitmap(LED_OFF, LED3->Picture->Bitmap);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Timer1Timer(TObject *Sender)
{
  io_buffer.binary0 = DigitalOut1Box->Checked;       // store digital outputs
  io_buffer.binary1 = DigitalOut2Box->Checked;       // into io_buffer
  io_buffer.binary2 = DigitalOut3Box->Checked;

  if (xferpipe(&io_buffer)) {                        // transmit & receive
    StatusBar->SimpleText = "USB08 Hardware found."; // xfer was OK

    if (io_buffer.binary0)                           // update digital inputs
      Images->GetBitmap(LED_ON, LED1->Picture->Bitmap);
    else
      Images->GetBitmap(LED_OFF, LED1->Picture->Bitmap);

    if (io_buffer.binary1)
      Images->GetBitmap(LED_ON, LED2->Picture->Bitmap);
    else
      Images->GetBitmap(LED_OFF, LED2->Picture->Bitmap);

    if (io_buffer.binary2)
      Images->GetBitmap(LED_ON, LED3->Picture->Bitmap);
    else
      Images->GetBitmap(LED_OFF, LED3->Picture->Bitmap);

    LED1->Refresh();                         // refresh LED-images
    LED2->Refresh();
    LED3->Refresh();

    Analog1->Progress = io_buffer.analog0;   // update analog-values
    Analog2->Progress = io_buffer.analog1;
    Analog3->Progress = io_buffer.analog2; }
  else                                       // xfer FAILED
    StatusBar->SimpleText = "Hardware Error or no Hardware detected!";
}
//---------------------------------------------------------------------------

#ifdef UseUSBpipe

//---------------------------------------------------------------------------
//----- The following code will be compiled if the "UseUSBpipe" label  ------
//----- is defined.                                                    ------
//----- Data transfer will happen over USB port. Make sure that the    ------
//----- "usbio.sys" driver is installed correctly!                     ------
//---------------------------------------------------------------------------

bool TMainForm::openpipe(void)
{
  const GUID g_UsbioID = USBIO_IID;      // globally unique identifier for
                                         // "usbio.sys"-driver (see Win32 SDK)
  USB_DEVICE_DESCRIPTOR desc;
  USBIO_SET_CONFIGURATION conf;

  conf.ConfigurationIndex = 0;           // select configuration 0 on USB08-board
  conf.NbOfInterfaces = 1;               // one interface to configure
  conf.InterfaceList[0].InterfaceIndex = 0;         // settings of this
  conf.InterfaceList[0].AlternateSettingIndex = 0;  // interface
  conf.InterfaceList[0].MaximumTransferSize = 8;

  HDEVINFO g_DevList = CUsbIo::CreateDeviceList(&g_UsbioID); // creating a list of
                                                             // all devices connected
                                                             // to USB
  if (g_DevList != NULL)
    for (int i = 0; i < 127; i++)                   // scan USB for our HC08-board
      if (!UsbIo.Open(i, g_DevList, &g_UsbioID)) {  // try to open device
        if (!UsbIo.GetDeviceDescriptor(&desc))      // get descriptor
          if (desc.idVendor == HC08_ID_VENDOR &&    // check for match of vendor
             desc.idProduct == HC08_ID_PRODUCT)     // and product IDs
            return !UsbIo.SetConfiguration(&conf) &&                     // configure device
                   !UsbIoPipeIn.Bind(i, 0x81, g_DevList, &g_UsbioID) &&  // bind in and
                   !UsbIoPipeOut.Bind(i, 0x02, g_DevList, &g_UsbioID); } // out pipes
      else
        break;
  return FALSE;          // if UsbIo.Open failed, stop scanning USB
}
//---------------------------------------------------------------------------
bool TMainForm::xferpipe(Tio_buffer *io_buffer)
{
  unsigned long ByteCount;

  return !UsbIoPipeOut.WriteSync(io_buffer, ByteCount = sizeof(Tio_buffer), 1000) &&
         ByteCount == sizeof(Tio_buffer) &&
         !UsbIoPipeIn.ReadSync(io_buffer, ByteCount = sizeof(Tio_buffer), 1000) &&
         ByteCount == sizeof(Tio_buffer);
}
//---------------------------------------------------------------------------
bool TMainForm::closepipe(void)
{
  UsbIoPipeIn.Close();
  UsbIoPipeOut.Close();
  UsbIo.Close();

  return TRUE;                  // Close is always TRUE
}
//---------------------------------------------------------------------------

#else

//---------------------------------------------------------------------------
//----- The following code will be compiled if the "UseUSBpipe" label  ------
//----- is NOT defined.                                                ------
//----- Data transfer will happen over serial port. Default value for  ------
//----- port is COM1 and for baud rate 2400. Your're able to override  ------
//----- these settings by creating a "IO08.INI" file in your WINDOWS-   ------
//----- directory with a "[COM Settings]"-section and the lines        ------
//----- "comport=COMx" and "baudrate=xxxx".                            ------
//---------------------------------------------------------------------------

bool TMainForm::openpipe(void)
{
  AnsiString     comport;
  unsigned long  baudrate;
  DCB            DCB;	                // device-control block
  COMMTIMEOUTS   CommTimeouts;          // com-timeout structure

  TIniFile *IniFile = new TIniFile("IO08.INI"); // open .ini-file

  comport = IniFile->ReadString("COM Settings", "comport", "COM1");
  baudrate = IniFile->ReadInteger("COM Settings", "baudrate", 2400);

  delete IniFile;

  MyHandle = CreateFile(comport.c_str(),        // COMport from .ini-file
    GENERIC_READ | GENERIC_WRITE,               // read/write-mode
    0,                                          // must be opened w/exclusive-access
    NULL,                                       // no security attributes
    OPEN_EXISTING,                              // creation distribution
    0,                                          // no overlapped I/O
    NULL                                        // hTemplate must be NULL
    );

  GetCommState(MyHandle, &DCB);

  DCB.BaudRate        = baudrate;
  DCB.fBinary         = TRUE;
  DCB.fParity         = FALSE;
  DCB.fOutxCtsFlow    = false;                  // disable flow-control for
  DCB.fOutxDsrFlow    = false;                  // work with only 3 lines
  DCB.fDsrSensitivity = false;                  // (GND, RXD, TXD)
  DCB.fOutX           = FALSE;
  DCB.fInX            = FALSE;
  DCB.fNull           = FALSE;
  DCB.fRtsControl     = RTS_CONTROL_DISABLE;    // disable flow control
  DCB.fAbortOnError   = FALSE;
  DCB.ByteSize        = 8;
  DCB.StopBits        = ONESTOPBIT;

  CommTimeouts.ReadIntervalTimeout         = 0;    // build TimeOut-Structure
  CommTimeouts.ReadTotalTimeoutMultiplier  = 50;
  CommTimeouts.ReadTotalTimeoutConstant    = 0;
  CommTimeouts.WriteTotalTimeoutMultiplier = 0;
  CommTimeouts.WriteTotalTimeoutConstant   = 0;

  if (MyHandle != INVALID_HANDLE_VALUE)
    return SetCommState(MyHandle, &DCB) &&            // set DCB
           SetCommTimeouts(MyHandle, &CommTimeouts);  // set timeouts
  else
    return false;                                     // open failed
}
//---------------------------------------------------------------------------
bool TMainForm::closepipe(void)
{
  if (MyHandle != INVALID_HANDLE_VALUE)
    return CloseHandle(MyHandle);                  // close succeeded
  else
    return false;                                  // close failed
}
//---------------------------------------------------------------------------
bool TMainForm::xferpipe(Tio_buffer *io_buffer)
{
  unsigned long ByteXfered;

  if (MyHandle != INVALID_HANDLE_VALUE)               // try only with valid handle
    return WriteFile(MyHandle, io_buffer, sizeof(Tio_buffer), &ByteXfered, NULL) &&
           ByteXfered == sizeof(Tio_buffer) &&
           ReadFile(MyHandle, io_buffer, sizeof(Tio_buffer), &ByteXfered, NULL) &&
           ByteXfered == sizeof(Tio_buffer);
  else
    return false;
}

#endif

